<?php declare(strict_types=1);

namespace App\Controller;

use App\Entity\PreviouslyOwnedCpu;
use App\Form\PreviouslyOwnedCpuType;
use App\Repository\CpuRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\
{RedirectResponse, Request, Response};
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Routing\Annotation\Route;

#[Route('previously_owned_cpu')]
class PreviouslyOwnedCpuController extends AbstractController {
	use FormControllerBase;

	protected const ENTITY        = PreviouslyOwnedCpu::class;
	protected const TEMPLATE_PATH = 'previously_owned_cpu/';
	protected const ROUTE_PREFIX  = 'previously_owned_cpu_';
	protected const FORM          = PreviouslyOwnedCpuType::class;

	public function __construct(private readonly EntityManagerInterface $entityManager)
	{
	}

	#[Route('/', name: 'previously_owned_cpu_index', methods: ['GET'])]
	public function index(): Response
	{
		return $this->itemListView('cpus');
	}

	#[Route('/{id}', name: 'previously_owned_cpu_show', methods: ['GET'])]
	public function show(PreviouslyOwnedCpu $previouslyOwnedCpu): Response
	{
		return $this->itemView($previouslyOwnedCpu, 'previously_owned_cpu');
	}

	#[Route('/{id}/edit', name: 'previously_owned_cpu_edit', methods: ['GET', 'POST'])]
	public function edit(Request $request, PreviouslyOwnedCpu $previouslyOwnedCpu): Response
	{
		return $this->itemUpdate($request, $previouslyOwnedCpu, 'previously_owned_cpu');
	}

	#[Route('/{id}', name: 'previously_owned_cpu_delete', methods: ['POST'])]
	public function delete(Request $request, PreviouslyOwnedCpu $previouslyOwnedCpu): Response
	{
		return $this->deleteCSRF($request, $previouslyOwnedCpu);
	}

	/**
	 * Moves a Gpu to the previouslyOwned table
	 */
	#[Route(path: '/{id}/reacquire', name: 'previously_owned_cpu_reacquire', methods: ['POST'])]
	public function reacquireAction(Request $request, PreviouslyOwnedCpu $Cpu): RedirectResponse
	{
		return $this->itemReacquire($request, $Cpu, 'cpu_index');
	}

	/**
	 * Creates a form to move
	 *
	 * @param PreviouslyOwnedCpu $Cpu The Cpu entity
	 */
	private function createReacquireForm(PreviouslyOwnedCpu $Cpu): FormInterface
	{
		return $this->buildForm($Cpu, 'previously_owned_cpu_reacquire', 'POST');
	}
}
